@@ -39,7 +39,7 @@ use constant ERROR_58_HINTS => q{Non-subroutine %s hints for %s are not supporte
 use constant MIN_IPC_SYS_SIMPLE_VER => 0.12;
 
 # All the Fatal/autodie modules share the same version number.
-our $VERSION = '2.05';
+our $VERSION = '2.06';
 
 our $Debug ||= 0;
 
@@ -100,6 +100,7 @@ my %TAGS = (
     ':2.03'  => [qw(:default)],
     ':2.04'  => [qw(:default)],
     ':2.05'  => [qw(:default)],
+    ':2.06'  => [qw(:default)],
 );
 
 $TAGS{':all'}  = [ keys %TAGS ];
@@ -1086,10 +1087,19 @@ sub _make_fatal {
 
                 my \$caller_level = 0;
 
-                while ( (caller \$caller_level)[1] =~ m{^\\(eval \\d+\\)\$} ) {
+                my \$caller;
+
+                while ( (\$caller = (caller \$caller_level)[1]) =~ m{^\\(eval \\d+\\)\$} ) {
+
+                    # If our filename is actually an eval, and we
+                    # reach it, then go to our autodying code immediatately.
+
+                    goto &\$code if (\$caller eq \$filename);
                     \$caller_level++;
                 }
 
+                # We're now out of the eval stack.
+
                 # If we're called from the correct file, then use the
                 # autodying code.
                 goto &\$code if ((caller \$caller_level)[1] eq \$filename);
